function [post nLogLik] = inferMultiLogitGP_bohning(hyp, mean, cov, x, y, nClass, options)
% inference for Multinomial logit GP using Bohning's bound
% TODO Gradients with respect to parameters
% Written by Emtiyaz, CS, UBC, 
% Modified on Sep 13, 2011

  [maxItersInfer, lowerBoundTol, display, boundParams] = myProcessOptions(options, 'maxItersInfer', 1000, 'lowerBoundTol',1e-6, 'display', 1, 'boundParams', []);
  [N] = size(x,1);
  M = nClass-1;

  % compute the prior mean and covariance
  mean_ = meanFuncMultiClass(mean, nClass, hyp.mean, x);
  covMat = covFuncMultiClass(cov, nClass, hyp.cov, x);

  % form sA matrix (square root of A)
  A1 = (eye(M) - ones(M,M)/(M + 1))/2;
  if isempty(boundParams)
    sA = sparse([]);
    sA1 = chol(A1);
    for n = 1:N
      sA = blkdiag(sA,sA1);
    end
    A = sA'*sA;
  else
    sA = boundParams.sA;
    A = boundParams.A;
  end

  % compute covMat
  U = chol(eye(N*M) + sA*covMat*sA');
  T = U'\(sA*covMat);
  V = covMat - T'*T;

  % V*covMatInv*mean_
  T1 = U'\sA;
  t = T1*mean_;
  infrm = mean_ - T'*t;

  % variational inference
  m = full(mean_);
  T2 = sA'*(U\T);
  Dkl = 0.5*(2*sum(log(diag(U))) - trace(T'*T1));
  traceAV = -0.5*trace(A*V); 
  for i = 1:maxItersInfer
    % compute mean
    Psi = reshape(m,M,N);
    lse1 = log(sum(exp([Psi; zeros(1,N)])));
    diff = bsxfun(@minus, Psi, lse1);
    G = exp(diff);
    B = A1*Psi-G; 
    m = V*(y+B(:)) + infrm;

    % log Marg Lik
    Psi = Psi(:);
    alpha = (eye(M*N) - T2)*(y+B(:)) + T1'*t; 
    c = 0.5*Psi'*A*Psi - G(:)'*Psi + sum(lse1);
    logMargLik(i) = -( Dkl+ 0.5*(m-mean_)'*alpha )...
        + y'*m + traceAV - 0.5*m'*A*m + B(:)'*m - c;

    % convergence
    [converged, incr] = isConverged(logMargLik, 1e-4, 'objFun');
    if display
      fprintf('iter %d negLogLik %f\n', i, -logMargLik(i));
    end
    if incr < -1e-10 
      warning('Lower bound decreased');
      pause
    end
    if converged; break; end;
    if i == maxItersInfer
      warning('Maximum iterations reached');
    end
  end
  nLogLik = -logMargLik(end);

  % posterior distribution
  post.mean = m;
  post.covMat = V;
  post.varParams = Psi(:);

  % quantities needed for prediction
  post.alpha = alpha;
  post.sLambda = sA;
  post.U = U;

