function [post nLogLik out] = inferMultLogitGPReg_delta1(hyp, mean, cov, x, y, nClass, options)
% inference for Multinomial logit GP using the first-order delta method
% TODO Gradients with respect to parameters
% Written by Emtiyaz, CS, UBC, 
% Modified on Sep 13, 2011

  [maxItersInfer, lowerBoundTol, display] = myProcessOptions(options, 'maxItersInfer', 1000, 'lowerBoundTol',1e-6, 'display', 1);
  [N] = size(x,1);
  M = nClass-1;

  % minfunc options
  optMinFunc = struct('Display', display,...
                      'Method', 'lbfgs',...
                      'DerivativeCheck', 'on',...
                      'LS', 2,...
                      'MaxIter', maxItersInfer,...
                      'MaxFunEvals', maxItersInfer,...
                      'TolFun', lowerBoundTol,...
                      'TolX', lowerBoundTol);

  % convert data to dummy encoding
  y = encodeDataOneOfM(y, nClass*ones(N,1), 'M+1');

  % compute the prior mean and covariance
  covMat = zeros(N*M,N*M);
  mean_ = zeros(N*M,1);
  for k = 1:M
    idx = [k:M:(N-1)*M+k];
    hyp1.cov = [hyp.cov(1) hyp.cov(k+1)];
    covMat(idx,idx) = feval(cov{:}, hyp1.cov, x);
    mean_(idx) = feval(mean{:}, hyp.mean, x);
  end
  covMat = sparse(covMat);
  mean_ = sparse(mean_);

  % compute blkdiag indicator
  I = ones(M,M);
  blkDiagInd = [];
  for i = 1:N
    blkDiagInd = blkdiag(blkDiagInd, I);
  end
  blkDiagInd = sparse(blkDiagInd);

  % optimize
  v0 = rand(M*N,1);
  [v,nLogLik,exitFlag,inform]=minFunc(@funObj_deltaMultiLogitGP, v0, optMinFunc, y, mean_, covMat, blkDiagInd, M, N);
  alpha = v;

  % compute m
  m = mean_ + covMat*alpha;

  % compute V
  Mean = reshape(m,M,N);
  lse1mk = logsumexp([Mean; zeros(1,N)],1);
  g = exp(bsxfun(@minus, Mean, lse1mk));
  H = blkDiagInd; % preallocate memory as blk sparse matrix 
  UH = blkDiagInd;
  for n = 1:N
    idx = (n-1)*M + 1:n*M;
    Hn = diag(g(:,n)) - g(:,n)*g(:,n)';
    Hn = Hn + diag(max(diag(Hn), 1e-10));
    UHn = cholinc(sparse(Hn), struct('droptol',1e-10));
    H(idx,idx) = Hn;
    UH(idx,idx) = UHn;
  end
  B = eye(N*M) + UH*covMat*UH';
  U = chol(B);
  A = UH\(U\(U'\UH));
  V = A*covMat;

  % posterior distribution
  post.mean = m;
  post.covMat = V;

  % quantities needed for prediction
  post.alpha = alpha;
  post.sLambda = UH;
  post.U = U;

  % other information
  out.message = inform.message;
  out.nLogLiks = inform.trace.fval;

