function [post nLogLik out] = inferMultLogitGP_log(hyp, mean, cov, x, y, nClass, options)
% inference for Multinomial logit GP using bound on log function
% TODO Gradients with respect to parameters
% Written by Emtiyaz, CS, UBC, 
% Modified on Sep 13, 2011

  [maxItersInfer, lowerBoundTol, display] = myProcessOptions(options, 'maxItersInfer', 1000, 'lowerBoundTol',1e-6, 'display', 1);
  [N] = size(x,1);
  M = nClass-1;

  % minfunc options
  optMinFunc = struct('Display', display,...
                      'Method', 'lbfgs',...
                      'DerivativeCheck', 'off',...
                      'LS', 2,...
                      'MaxIter', maxItersInfer,...
                      'MaxFunEvals', maxItersInfer,...
                      'TolFun', lowerBoundTol,...
                      'TolX', lowerBoundTol);

  % compute the prior mean and covariance
  mean_ = meanFuncMultiClass(mean, nClass, hyp.mean, x);
  covMat = covFuncMultiClass(cov, nClass, hyp.cov, x);

 % optimize
  v0 = [zeros(M*N,1); 0*ones(M*N,1)];
  [v,nLogLik,exitFlag,inform]=minFunc(@funObj_logBoundMultiLogitGP, v0, optMinFunc, y, mean_, covMat, M, N);
  alpha = v(1:M*N);
  lambda = exp(v(M*N+1:end));

  % compute m, V and A
  m = mean_ + covMat*alpha;
  slambda = sqrt(lambda);
  B = eye(N*M) + diag(slambda)*covMat*diag(slambda);
  U = chol(B);
  Uinv = U\eye(M*N);
  A = diag(1./slambda)*Uinv*Uinv'*diag(slambda);
  V = A*covMat;

  % posterior distribution
  post.mean = m;
  post.covMat = V;

  % quantities needed for prediction
  post.alpha = alpha;
  post.sLambda = diag(slambda);
  post.U = U;

  % other information
  out.message = inform.message;
  out.nLogLiks = inform.trace.fval;
