function [post nLogLik g out] = inferStickLogitGP_pw(hyp, mean, cov, x, y, nClass, options)
% inference for Stick breaking logit GP using piecewise bounds
% TODO Gradients with respect to parameters
% Written by Emtiyaz, CS, UBC, 
% Modified on Sep 13, 2011

  [maxItersInfer, lowerBoundTol, display, v0, boundParams, ind] = myProcessOptions(options, 'maxItersInfer', 1000, 'lowerBoundTol',1e-6, 'display', 1, 'v0', [], 'boundParams', [], 'ind', []);
  [N] = size(x,1);
  M = nClass-1;

  % minfunc options
  optMinFunc = struct('Display', display,...
                      'Method', 'lbfgs',...
                      'DerivativeCheck', 'off',...
                      'LS', 2,...
                      'MaxIter', maxItersInfer,...
                      'MaxFunEvals', maxItersInfer,...
                      'TolFun', lowerBoundTol,...
                      'TolX', lowerBoundTol);

  % indicator for I(k<=y_n), vector of length M*N
  if isempty(ind)
    ind = reshape(y, M, N);
    ind = ~cumsum(ind,1) + ind;
    ind = ind(:);
  end

  % compute the prior mean and covariance
  mean_ = meanFuncMultiClass(mean, nClass, hyp.mean, x);
  covMat = covFuncMultiClass(cov, nClass, hyp.cov, x);

  % bound parameters
  if isempty(boundParams)
    boundParams = getPieceWiseBound('quad', 20);
  end

  % optimize
  if isempty(v0)
    v0 = [zeros(M*N,1); 0*ones(M*N,1)];
  end
  [v,nLogLik,exitFlag,inform]=minFunc(@funObj_stickLogitGP, v0, optMinFunc, y, mean_, covMat, M, N, boundParams, ind);
  alpha = v(1:M*N);
  lambda = exp(v(M*N+1:end));

  % compute m, V and A
  m = mean_ + covMat*alpha;
  slambda = sqrt(lambda);
  B = eye(N*M) + diag(slambda)*covMat*diag(slambda);
  U = chol(B);
  Uinv = U\eye(M*N);
  A = diag(1./slambda)*Uinv*Uinv'*diag(slambda);
  V = A*covMat;
 
  % posterior distribution
  post.mean = m;
  post.covMat = V;

  % quantities needed for prediction
  post.alpha = alpha;
  post.sLambda = diag(slambda);
  post.U = U;

  % gradient
  if nargout>2

    % gradient log-lik term y'*m + \sum_n pwbound(m,v)
    [f, gm, gv] = funObj_pw(m, diag(V), boundParams);
    gm = y - ind.*gm;
    gv = -ind.*gv;

    if isempty(hyp.cov); 
      gThetaC = []; 
    else
      for k = 1:M
        for j = 1:length(hyp.cov(k,:))
          Gkj = covFuncMultiClass(cov, nClass, hyp.cov, x, [], k, j);
          gThetaC(k,j) = alpha'*Gkj*gm + diag(A*Gkj*A')'*gv...
                   +0.5*(trace(A*diag(lambda)*Gkj) - trace(A*Gkj*diag(lambda)*A) - alpha'*Gkj*alpha);
        end
      end
    end
    if isempty(hyp.mean); 
      gThetaM = []; 
    else
      for k = 1:M
        for j = 1:length(hyp.mean(k,:))
          gkj = meanFuncMultiClass(mean, nClass, hyp.mean, x, k, j);
          gThetaM(k,j) = -alpha(:)'*gkj(:);
        end
      end
    end
    g = -[gThetaM(:); gThetaC(:)];
  end

  % other information
  if nargout >3
    out.message = inform.message;
    out.nLogLiks = inform.trace.fval;
  end


