function phat = predProbMultiLogitGP(mu_s, Sigma_ss, Sigma_s, post, nClass, S)
% Return predictive probilities for multiLogitGP
% This function samples from the posterior predictive distribution and pass it
% through softmax function to get pred probs
%
% nClass is the number of classes
% Let M = nClass -1, then
% mu_s is mu(x*) of size (Ntest*M)x1
% Sigma_ss is Sigma(x*,x*) of size (NtestxM)x1
% Sigma_s is Sigma(x*,x) of size (NtestxM)x(NtrainxM),
% post is posterior distribution with fields {alpha, sLambda, U}, 
% see getPredictDist*GP function for details
%
% Written by Emtiyaz, CS, UBC
% Modified on September 20, 2011

  % get quantites for prediction
  alpha = post.alpha;
  U = post.U;
  sLambda = post.sLambda;
  M = nClass-1;
  Ntest = length(mu_s)/M;

  % for each test example
  for i = 1:Ntest
    idx = [(i-1)*M+1:i*M];
    % posterior distribution of z* 
    [m_s, V_s] = getPredictDistMultiClassGP(mu_s(idx), diag(Sigma_ss(idx)), Sigma_s(idx,:), alpha, U, sLambda);
    % sample, pass through likelihood and average
    z = mvnrnd(m_s(:)', V_s, S)';
    z = [z; zeros(1,S)];
    prob = exp(bsxfun(@minus, z, log(sum(exp(z)))));
    phat(:,i) = sum(prob,2)/S;
  end

