function [f,g] = funObj_maxParams_stickLogitLGM(v, ss, Ybitvec, wY, ind, nCd, boundParams)
% Objective function for Mstep to learn (Wd,w0d) for stickLogitLGM for d'th
% dimension using piecewise bound
% f = sum_n y_dn'*mbar_dn - B(mbar_dn, vbar_dn/2, alpha_dn)
% where, mbar_dn := Wd*mn + w0d, vbar_dn := diag(Wd'*Vn*Wd)
% 'n' is the set of observations where this dimension was observed.
% g = sum_n (y_dn - gm_dn)*mn' - 2*W_d*diag(gv_dn)*V_n
% where gm_dn, gv_dn are gradients of the boundParams wrt to mbar_dn and vbar_dn
%
% Written by Emtiyaz, CS, UBC
% Modified on Dec. 5, 2011

  % get posterior mean and covariance
  M = ss.meanPost; 
  V = ss.covMatPost; 

  % get N,L
  [L N] = size(M);

  % get W w0
  W = v(1:L*nCd);
  W = reshape(W,nCd,L);
  w0 = v(L*nCd+1:end);

  % mbar and vbar
  mbar = bsxfun(@plus, W*M, w0);
  vbar = zeros(size(mbar));
  for n = 1:N
    vbar(:,n) = diag(W*V(:,:,n)*W');
  end

  % first term
  Ynew = bsxfun(@times, wY(:)', Ybitvec);
  f = Ynew(:)'*mbar(:);
  gW = Ynew*M';
  gw0 = sum(Ynew,2);

  % boundParams derivative wrt mbar and vbar 
  [fb, gmb, gvb] = funObj_pw(mbar(:), vbar(:), boundParams);
  fb = reshape(fb, nCd, N);
  gmb = reshape(gmb, nCd, N);
  gvb = reshape(gvb, nCd, N);
  f = f - sum(wY.*sum(ind.*fb,1));
  wgmb = bsxfun(@times, wY, ind.*gmb);
  wgvb = bsxfun(@times, wY, ind.*gvb);
  gW = gW - wgmb*M';
  for n = 1:N
    gW = gW - 2*diag(wgvb(:,n))*W*V(:,:,n);
  end
  gw0 = gw0 - sum(wgmb,2);

  g = [gW(:); gw0];
  f = -f;
  g = -full(g);

