function [f,g]=funObj_multiLogitLGM_gradient(v, Y, params, funObjBound, boundp)
% Objective function for inference in multiLogitLGM using bound on the LSE
% function specified by funObjBound
%
% TODO significant speedup can be obtained if multiplication with W can be
% simplified (in computation of mbar, vbar and contribution from the bound),
% for example in classification.
% TODO add boundp optimization
% Written by Emtiyaz, CS, UBC
% Modified on Dec. 5, 2011

  nClass = params.others.nClass;

  % get parameters
  W = params.loadingMat;
  w0 = params.bias;
  Mu = params.mean;
  invSigma = params.precMat;
  logdetSigma = params.logDetCovMat;
  L = length(params.mean);
  D = length(nClass);

  %Extract mean and covariance
  M = v(1:L);
  M = M(:);
  idx = L + [1:L*(L+1)/2];
  V = unpackcovariance(v(idx),L);
  % only for regression model
  if length(v)>L+L*(L+1)/2
    estimateRegBias = 1;
    beta0 = v(L+L*(L+1)/2+1:end);
    w0 = w0 + repmat(beta0,D,1);
  else
    estimateRegBias = 0;
  end

  % initialize
  f=0;
  gM = zeros(size(M));
  gV = zeros(size(V));

  %Check if current coavriance matrix is positive definite
  [U,posdef] = cholproj(V);
  if ~posdef
    f = -inf;
    g = 0;
    return  
  end

  % compute determinant and inverse
  logdetV = 2*sum(log(diag(U)));
  invV = solve_chol(U, eye(L));%U\(U'\eye(L));

  %Contribution from the KL divergence term
  MMu = M-Mu;
  invSigmaMMu = invSigma*MMu;
  f = f - 0.5*(logdetSigma-logdetV + V(:)'*invSigma(:) + MMu'*invSigmaMMu - L);
  gM = gM - invSigmaMMu;
  gV = gV + 0.5*(invV-invSigma);

  % compute mbar and vbar
  mbar = W*M + w0;
  vbar = sum(W.*(V*W')',2); % W*V*W' efficient

  %Contribution from the first term in the likelihood 
  f = f + Y'*mbar;
  gM = gM + W'*Y;

  % contribution from the bound
  [fb, gmb, gvb, galpha] = funObjBound(mbar, vbar, boundp, nClass-1);
  f = f - sum(fb);
  gM = gM - W'*gmb;
  gV = gV - W'*bsxfun(@times, gvb(:), W); % W'*diag(gvb)*W efficient

  % add double contribution for off-diagonal elements 
  gV = gV + triu(gV,1);
  g=[gM(:); gV(triu(ones(L))==1); galpha];

  % add regression bias 
  if estimateRegBias
    K = unique(nClass);
    g = [g; sum(reshape(Y-gmb, K-1, D),2)];
  end

  % return
  g=-g;
  f=-f;

