function [f, gm, gv, gbp]=funObj_multiLogitLGM_log(mbar, vbar, boundp, K)
% Log (Blei's bound or first-order delta method) bound for multilogit likelihood
  
  if length(unique(K)) == 1
    % if number of categories are same and there is only one observation, computation can be done in parallel
    [f, gm, gv]=funObj_LSE_log_parallel(mbar, vbar, K);
  else
    % otherwise loop over each category
    [f, gm, gv]=funObj_LSE_log_serial(mbar, vbar, K);
  end
  gbp = [];

function [f, gm, gv]=funObj_LSE_log_serial(mbar, vbar, K)
% serial version when number of categories are different 

  D = length(K); 
  f = zeros(D,1);
  gm = zeros(size(mbar));
  gv = zeros(size(vbar));

  for d = 1:D
    idx = sum(K(1:d-1))+1:sum(K(1:d));
    m_d = mbar(idx);
    v_d = vbar(idx);
    x = m_d + v_d/2;
    f(d) = logsumexp([x(:);0]);
    g = exp(x-f(d));
    gm(idx) = g;
    gv(idx) = 0.5*g;
  end

function [f, gm, gv]=funObj_LSE_log_parallel(mbar, vbar, K)
% parallel version when number of categories are same

  D = length(K); 
  K = unique(K);
  x = mbar + vbar/2;
  x = reshape(x, K, D);
  lse = logsumexp([x; zeros(1,D)]);
  g = bsxfun(@minus, x, lse);
  g = exp(g);
  f = lse;
  g = g(:);
  gm = g(:);
  gv = 0.5*g(:);


