function [f,g]=funObj_stickLogitLGM_pw(v, data, params, bound)
% Objective function for inference in stickLogitLGM using piecewise bound
%
% Written by Emtiyaz, CS, UBC
% Modified on Dec. 5, 2011

  Y = data.Y;
  ind = data.ind;

  % get parameters
  W = params.loadingMat;
  w0 = params.bias;
  Mu = params.mean;
  invSigma = params.precMat;
  logdetSigma = params.logDetCovMat;
  nClass = params.others.nClass;
  L = length(params.mean);
  D = length(nClass);

  %Extract mean and covariance
  M = v(1:L);
  M = M(:);
  idx = L + [1:L*(L+1)/2];
  V = unpackcovariance(v(idx),L);
  % only for regression model
  if length(v)>L+L*(L+1)/2
    estimateRegBias = 1;
    beta0 = v(L+L*(L+1)/2+1:end);
    w0 = w0 + repmat(beta0,D,1);
  else
    estimateRegBias = 0;
  end

  % initialize
  f=0;
  gM = zeros(size(M));
  gV = zeros(size(V));

  %Check if current coavriance matrix is positive definite
  [U,posdef] = cholproj(V);
  if ~posdef
    f = -inf;
    g = 0;
    return  
  end

  % compute determinant and inverse
  logdetV = 2*sum(log(diag(U)));
  invV = solve_chol(U, eye(L));%U\(U'\eye(L));

  %Contribution from the KL divergence term
  MMu = M-Mu;
  invSigmaMMu = invSigma*MMu;
  f = f - 0.5*(logdetSigma-logdetV + V(:)'*invSigma(:) + MMu'*invSigmaMMu - L);
  gM = gM - invSigmaMMu;
  gV = gV + 0.5*(invV-invSigma);

  % mbar and vbar
  mbar = W*M + w0;
  vbar = sum(W'.*(V*W'),1)'; %vbar = diag(W*V*W');

  %Contribution from the first term in the likelihood 
  f = f + Y'*mbar;
  gM = gM + W'*Y;

  % contribution from the bound
  [fb, gmb, gvb] = funObj_pw(mbar, vbar, bound);
  gmb = ind.*gmb;
  gvb = ind.*gvb;
  f = f - sum(ind.*fb);
  gM = gM - W'*gmb;
  gV = gV - W'*bsxfun(@times, gvb(:), W); %gV = gV - W'*diag(gvb)*W;

  % add double contribution for off-diagonal elements 
  gV = gV + triu(gV,1);
  g=[gM(:); gV(triu(ones(L))==1)];

  % add regression bias 
  if estimateRegBias
    K = unique(nClass);
    g = [g; sum(reshape(Y-ind.*gmb, K-1, D),2)];
  end

  % return
  g=-g;
  f=-f;

