function [ss, logLik, postDist] = inferMultiLogitReg_bohning(data, params, funObj, options)

  [maxItersInfer, lowerBoundTol, initVec, display, computeSs, estimateMean, estimateCovMat, estimateLoadingMat, estimateBias] = myProcessOptions(options, 'maxItersInfer', 30, 'lowerBoundTol', 1e-6, 'initVec', [], 'display', 1, 'computeSs', 0, 'estimateMean', 0, 'estimateCovMat', 0, 'estimateLoadingMat', 0, 'estimateBias', 0);

  % get data
  Y = data.Y;
  Yunique = data.Yunique;
  wY = data.Ycount;
  Ybitvec = data.Ybitvec;
  miss = data.missing;
  J = data.mappingJ;
  I = data.mappingI;

  % get D,N,L,K
  [D,N] = size(Y);
  Nbitvec = size(Ybitvec,2);
  L=size(params.mean,1);
  K = params.others.nClass;
  nC = unique(K-1);
  Widx = params.others.loadingMatIdx;

  % YbitVec, remove the baseline categories
  Ybitvec = Ybitvec(Widx,:);

  % get parameters
  invSigma = params.precMat;
  logdetSigma = params.logDetCovMat;
  Mu = params.mean;
  W = params.loadingMat(Widx,:);% remove the baseline category
  w0 = params.bias(Widx,:);% remove the baseline category

  % bound params
  A = (eye(nC) - ones(nC,nC)/(nC + 1))/2;
  iA = inv(A);

  % compute posterior covariance using
  WAW = 0;
  for d = 1:D
    idx = (d-1)*nC+1:d*nC;
    WAW = WAW + W(idx,:)'*A*W(idx,:);
  end
  V = inv(invSigma + WAW);

  % precompute some quantities
  logdetV = logdet(V);
  trVinvSigma = V(:)'*invSigma(:);
  VSigmaInvMu = V*invSigma*Mu;
  trWAWV = 0;
  for d = 1:D
    idx = (d-1)*nC+1:d*nC;
    trWAWV = trWAWV + trace(A*W(idx,:)*V*W(idx,:)');
  end
  VW = V*W';

  % initialize
  if isempty(initVec)
    M = repmat(rand(length(Mu),1), 1, Nbitvec);
  else
    M = initVec;
  end
  beta0 = zeros(nC,1);
  Mdn = W*M + repmat(beta0,D,1);

  % inference
  for iter = 1:maxItersInfer
    % update variational parameter
    Psi = Mdn;

    % compute b
    Psi = reshape(Psi, nC, D);
    lse = logsumexp([Psi; zeros(1,D)],1); 
    G = exp(bsxfun(@minus, Psi, lse));
    APsi = A*Psi;
    B = APsi(:) - G(:);
    G = G(:);
    Psi = Psi(:);

    % Mean
    YplusB = Ybitvec+B;
    ypseudo = YplusB - repmat(A*beta0,D,1);
    M = bsxfun(@plus, VW*ypseudo, VSigmaInvMu);

    % beta0
    WM = W*M;
    beta0 = iA*sum(reshape(YplusB,nC,D),2) - sum(reshape(WM,nC,D),2);
    beta0 = beta0/D;

    % Mbar
    Mdn = WM + repmat(beta0,D,1);

    % log likelihood
    diff = bsxfun(@minus, M, Mu);
    Dkl = 0.5*(logdetSigma-logdetV + trVinvSigma - L...
                + sum(diff.*(invSigma'*diff),1));
    c = 0.5*sum(Psi.*(APsi(:))) - sum(G.*Psi) + sum(lse(:));
    AMdn = A*reshape(Mdn,nC,D);
    logLik_n = -Dkl + sum(YplusB.*Mdn) - 0.5*trWAWV - 0.5*sum(Mdn.*(AMdn(:))) - c;
    logLik(iter) = N*sum(wY.*logLik_n);

    % convergence
    [converged, incr] = isConverged(logLik, 1e-4, 'objFun');
    if display
      fprintf('iter %d logLik %f\n', iter, logLik(iter));
    end
    if converged; break; end;
    if i == maxItersInfer
      warning('Maximum iterations reached');
    end
    if incr < -1e-10 
      warning('Lower bound decreased');
    end
  end
  meanPost = M;
  covMatPost = V;
  logLik = logLik(end);

  % postDist
  if nargout > 2
    postDist.mean = meanPost;
    postDist.covMat = covMatPost;
    postDist.logLik_n = logLik_n;
    postDist.regBias = beta0;
  end

  ss = 0;
