function [params,options] = maxParamsStickLogitLGM_pw(ss, data, params, funObj, options)
% Mstep function for stickLogitLGM parameter learning 
%
%TODO add estimateBias option
% Written by Emtiyaz, CS, UBC
% Modified on Dec. 5, 2011

  [estimateLoadingMat, estimateBias, boundParams] = myProcessOptions(options, 'estimateLoadingMat', 0, 'estimateBias', 0, 'boundParams', []);

  % mean and CovMat
  [params, options] = maxParamsCatLGM_gaussianPrior(ss, data, params, options);

  % loadingMat and bias
  if estimateLoadingMat
    % minfunc options
    [maxItersMstep, lowerBoundTolMstep, displayMstep] = myProcessOptions(options, 'maxItersMstep', 30, 'lowerBoundTolMstep', 1e-6, 'displayMstep', 0);
    optMinFunc.Display = displayMstep;
    optMinFunc.DerivativeCheck = 'off';
    optMinFunc.Method = 'lbfgs';
    optMinFunc.LS = 2;
    optMinFunc.MaxIter = maxItersMstep;
    optMinFunc.MaxFunEvals = maxItersMstep;
    optMinFunc.TolFun = lowerBoundTolMstep;
    optMinFunc.TolX = lowerBoundTolMstep;

    % parameters
    nC = params.others.nClass-1;
    D = params.others.nDim;
    L = params.others.latentDim;
    Widx = params.others.loadingMatIdx;
    W = params.loadingMat(Widx,:);
    w0 = params.bias(Widx,:);

    % data
    Yunique = data.Yunique;
    wY = data.Ycount;
    Ybitvec = data.Ybitvec(Widx,:);
    ind = data.ind(Widx,:);

    % bound params
    if isempty(boundParams)
      boundParams = getPiecewiseBound('quad',20);
    end

    for d = 1:D
      % measurements where d'th dimension is observed
      obs = find(~isnan(Yunique(d,:)));
      % data vector
      idx = sum(nC(1:d-1))+1:sum(nC(1:d));
      Ybitvec_d = Ybitvec(idx,obs);
      ind_d = ind(idx,obs);
      wY_d = wY(obs);
      % sufficient statistics
      ss_d.meanPost = ss.meanPost(:,obs);
      ss_d.covMatPost = ss.covMatPost(:,:,obs);
      % initialize
      Wd = W(idx,:);
      w0d = w0(idx);
      v0 = [Wd(:); w0d(:)];
      % optimize
      [v,fn]=minFunc(@funObj_maxParams_stickLogitLGM, v0, optMinFunc, ss_d, Ybitvec_d, wY_d, ind_d, nC(d), boundParams);
      % store
      W(idx,:) = reshape(v(1:L*nC(d)), nC(d), L);
      w0(idx) = v(L*nC(d)+1:end);
    end
    params.loadingMat(Widx,:) = W;
    params.bias(Widx,:) = w0;
  end

  % initVec
  if isfield(ss, 'initVec')
    options.initVec = ss.initVec;
  end

